
#pragma once

#include "MidiWrapper.h"

#include <fluidsynth.h>

// libsmf - helper lib for reading midi files
#include <smf.h>

/**
  * class FluidsynthWrapper
  *
  */


class FluidsynthWrapper
{
public:
    FluidsynthWrapper();
    ~FluidsynthWrapper();
    
    // forbid copying
    FluidsynthWrapper(FluidsynthWrapper const&) = delete;
    FluidsynthWrapper& operator=(FluidsynthWrapper const&) = delete;
    
    static void seqCallback(unsigned int time, fluid_event_t* e, fluid_sequencer_t* seq, void* data);

    
    void ShallowInit();
    void DeepInit(MidiWrapper&);
    void Unload();
    
    // returns the samplerate that will be synthesized at
    unsigned int GetSampleRate();
    
    unsigned int GetVoices();
    unsigned int GetChannelsPerVoice();
    
    // returns the tick count the sequencer had during a call to this.Init()
    unsigned int GetInitTick();
    
    void AddEvent(smf_event_t* event, double offset=0.0);
    void ScheduleLoop(MidiLoopInfo* info);
    void FinishSong(int millisec);
    
    void Render(float* bufferToFill, frame_t framesToRender);

private:
    fluid_settings_t* settings = nullptr;
    fluid_synth_t* synth = nullptr;
    fluid_sequencer_t* sequencer = nullptr;
    
    fluid_event_t* synthEvent = nullptr;
    fluid_event_t* callbackEvent = nullptr;

    short synthId;
    Nullable<short> myselfID;
    
    // fluidsynth's synth has no samplerate getter, so cache it here
    unsigned int cachedSampleRate = 0;
    
    // tick count of the sequencer when this->Init() was called
    unsigned int initTick = 0;

    string cachedSf2;
    int cachedSf2Id = -1;
    
    void setupSettings();
    void setupSynth(MidiWrapper&);
    void setupSeq(MidiWrapper&);

    void deleteSynth();
    void deleteSeq();
};
